
import os
import time
import hmac
import base64
import hashlib
import collections


class BTCMException(Exception):
    pass


def build_headers(end_point, post_data=None):

    api_key = os.environ['BTCMARKETS_API_KEY'].encode("utf-8")
    secret = os.environ['BTCMARKETS_SECRET'].encode("utf-8")

    timestamp = str(int(time.time() * 1000))
    string_body = end_point + "\n" + timestamp + "\n"
    if post_data is not None:
        string_body += post_data

    b64_secret = base64.standard_b64decode(secret)
    rsig = hmac.new(b64_secret, string_body.encode("utf-8"), hashlib.sha512)
    bsig = base64.standard_b64encode(rsig.digest()).decode("utf-8")

    return collections.OrderedDict([
        ("Accept", "application/json"),
        ("Accept-Charset", "UTF-8"),
        ("Content-Type", "application/json"),
        ("apikey", api_key.decode("utf-8")),
        ("timestamp", timestamp),
        ("signature", bsig),
    ])


def maybe_list(x):
    if isinstance(x, (int, float, str, dict)):
        return [x]
    return x





class Instrument:
    BTC = 'BTC'
    ETH = 'ETH'
    LTC = 'LTC'
    ETC = 'ETC'


class Currency:
    AUD = 'AUD'
    BTC = 'BTC'


class OrderSide:
    Buy = 'Bid'
    Sell = 'Ask'


class OrderType:
    LIMIT = 'Limit'
    MARKET = 'Market'


class Multipliers:
    PRICE = 100000000
    VOLUME = 100000000







import json
from collections import OrderedDict
from btcmarkets.enums import Multipliers
from btcmarkets.compat import urllib_request
from btcmarkets.util import build_headers, BTCMException, maybe_list


class BTCMarkets:

    base_url = 'https://api.btcmarkets.net'

    def __init__(self, request_func=urllib_request, return_kwargs=False):
        self.request = request_func
        self.return_kwargs = return_kwargs

    def get_accounts(self):
        return self.process(method='GET', end_point='/account/balance', parse_output=True)

    def get_order_book(self, instrument, currency):
        return self.process(method='GET', end_point='/market/%s/%s/orderbook' % (instrument, currency))

    def get_market_trades(self, instrument, currency, since=0):
        return self.process(method='GET', end_point='/market/%s/%s/trades?since=%s' % (instrument, currency, since))

    def get_open_orders(self, instrument, currency, limit=100, since=0):
        data = OrderedDict([
            ('currency', currency), ('instrument', instrument), ('limit', limit), ('since', since),
        ])
        return self.process(method='POST', end_point='/order/open', data=data, result_key='orders', parse_output=True)

    def get_order_history(self, instrument, currency, limit=100, since=0):
        data = OrderedDict([
            ('currency', currency), ('instrument', instrument), ('limit', limit), ('since', since)
        ])
        return self.process(method='POST', end_point='/order/history', data=data, result_key='orders', parse_output=True)

    def get_trade_history(self, instrument, currency, limit=100, since=0):
        data = OrderedDict([
            ('currency', currency), ('instrument', instrument), ('limit', limit), ('since', since)
        ])
        end_point = '/order/trade/history'
        return self.process(method='POST', end_point=end_point, data=data, result_key='trades', parse_output=True)

    def get_order_detail(self, order_ids):
        data = OrderedDict([('orderIds', maybe_list(order_ids))])
        return self.process(method='POST', end_point='/order/detail', data=data, result_key='orders', parse_output=True)

    def insert_order(self, instrument, currency, order_side, price, volume, order_type):
        """
        :param instrument: {'BTC', 'ETH', 'LTC'}
        :param currency: {'BTC', 'AUD'}
        :param order_side: ('Bid', 'Ask')
        :param price: price for order.
        :param volume: volume for order.
        :param order_type: {'Limit', 'Market')
        :return:
        """
        price_precision = {'AUD': 2, 'BTC': 8}[currency]
        data = OrderedDict([
            ('currency', currency),
            ('instrument', instrument),
            ('price', round(price, price_precision)),
            ('volume', volume),
            ('orderSide', order_side),
            ('ordertype', order_type),
            ('clientRequestId', '1'),
        ])
        return self.process(method='POST', end_point='/order/create', data=data, parse_output=True)

    def delete_order(self, order_ids):
        """
        :param order_ids: list of order_ids
        :return:
        """
        data = OrderedDict([('orderIds', maybe_list(order_ids))])
        return self.process(method='POST', end_point='/order/cancel', data=data, parse_output=True)

    def parse_input_data(self, data):
        for upper in ('currency', 'instrument'):
            if upper in data:
                    data[upper] = data[upper].upper()
        if 'price' in data:
            data['price'] = int(data['price'] * Multipliers.PRICE)
        if 'volume' in data:
            data['volume'] = int(data['volume'] * Multipliers.VOLUME)
        return data

    @staticmethod
    def parse_output_data(data):
        for x in maybe_list(data):
            if isinstance(x, dict):
                if 'price' in x:
                    x['price'] = x['price'] / Multipliers.PRICE
                if 'volume' in x:
                    x['volume'] = x['volume'] / Multipliers.VOLUME
                if 'balance' in x:
                    x['balance'] = x['balance'] / Multipliers.VOLUME
        return data

    def build_request(self, method, end_point, data=None):
        url = '%s/%s' % (self.base_url, end_point)
        if data is not None:
            data = self.parse_input_data(data)
            data = json.dumps(data, separators=(',', ':'))
        headers = build_headers(end_point, data)
        return dict(method=method, url=url, headers=headers, data=data)

    def process(self, method, end_point, data=None, result_key=None, parse_output=True):
        kwargs = self.build_request(method, end_point, data)
        if self.return_kwargs:
            return kwargs
        resp = self.request(**kwargs)
        if isinstance(resp, dict) and resp.get('errorMessage') is not None:
            raise BTCMException('[%s] %s' % (resp['errorCode'], resp['errorMessage']))
        if result_key:
            resp = resp[result_key]
        if parse_output:
            resp = self.parse_output_data(resp)
        return resp



import time

import pytest

from btcmarkets.api import BTCMarkets
from btcmarkets.compat import HTTPError
from btcmarkets.enums import OrderSide, OrderType
from btcmarkets.util import BTCMException


def test_pair_specific_funcs(api):
    instrument, currency = 'BTC', 'AUD'
    pair_funcs = [
        api.get_open_orders,
        api.get_order_history,
        api.get_trade_history,
        api.get_market_trades,
    ]

    for func in pair_funcs:
        result = func(instrument=instrument, currency=currency)
        assert isinstance(result, list)


def test_order_book(api):
    result = api.get_order_book('BTC', 'AUD')
    assert all(x in result for x in ('bids', 'asks',))


def test_get_accounts(api):
    accounts = api.get_accounts()
    assert isinstance(accounts, list)
    print(accounts)


def test_order_insert_delete(api):
    instrument, currency = 'BTC', 'AUD'
    price = 1
    volume = 0.001

    # Insert a super low bid
    resp_insert = api.insert_order(
        instrument=instrument, currency=currency, order_side=OrderSide.Buy, price=price,
        volume=volume, order_type=OrderType.LIMIT
    )
    assert (resp_insert['id'])

    # Check order exists and is correct
    time.sleep(0.5)
    resp_info = api.get_order_detail(order_ids=[resp_insert['id']])
    assert resp_info[0]['status'] in ('New', 'Placed')
    assert resp_info[0]['volume'] == volume
    assert resp_info[0]['price'] == price

    # Check delete
    resp_delete = api.delete_order(order_ids=[resp_insert['id']])
    assert resp_delete['responses'][0]['id']
    assert resp_delete['responses'][0]['success']
    time.sleep(0.5)
    assert not api.get_open_orders(instrument, currency)


def test_return_kwargs():
    api = BTCMarkets(return_kwargs=True)
    kwargs = api.get_accounts()
    assert kwargs['url'] == 'https://api.btcmarkets.net//account/balance'
    assert kwargs['method'] == 'GET'
    assert kwargs['headers']


def test_process_exception(api):
    with pytest.raises(HTTPError):
        api.delete_order('a')


def test_btcmarkets_exception(api):
    with pytest.raises(BTCMException):
        api.get_trade_history('BTC', 'AUD', limit=99999)


def test_order_insert_currency_precision(api):
    success_orders = [
        {"instrument": "BTC", "currency": "AUD", "price": 0.1, "volume": 1,
         "order_side": "Bid", "order_type": "Limit"},
        {"instrument": "ETH", "currency": "BTC", "price": 0.00000001, "volume": 1,
         "order_side": "Bid", "order_type": "Limit"},
        {"instrument": "BTC", "currency": "AUD", "price": 0.011, "volume": 1,
         "order_side": "Bid", "order_type": "Limit"},

    ]
    for order in success_orders:
        resp = api.insert_order(**order)
        assert resp['success']
        api.delete_order([resp['id']])



"""
_setup.py_

Cirrus template setup.py that reads most of its business
from the cirrus.conf file. This lightweight setup.py should be used by
projects managed with cirrus.

"""
import setuptools
try:
    import ConfigParser as configparser
except ImportError:
    import configparser


def get_default(parser, section, option, default):
    """helper to get config settings with a default if not present"""
    try:
        result = parser.get(section, option)
    except (configparser.NoSectionError, configparser.NoOptionError):
        result = default
    return result


# Build a parser and fetch setup options
parser = configparser.RawConfigParser()
parser.read('cirrus.conf')
src_dir = get_default(parser, 'package', 'find_packages', '.')
excl_dirs = get_default(parser, 'package', 'exclude_packages', [])
requirements_filename = get_default(
    parser,
    'build',
    'requirements-file',
    'requirements.txt'
)
requirements_file = open(requirements_filename)

# Manually parse the requirements file. Pip 1.5.6 to 6.0 has a function
# behavior change for pip.req.parse_requirements. You must use the setuptools
# format when specifying requirements.
#  - https://pythonhosted.org/setuptools/setuptools.html#declaring-dependencies
# Furthermore, you can't use line continuations with the following:
requirements = requirements_file.read().strip().split('\n')

setup_args ={
    'description': parser.get('package', 'description'),
    'include_package_data': True,
    'install_requires': requirements,
    'name': parser.get('package', 'name'),
    'version': parser.get('package', 'version'),
    'url': get_default(parser, 'package', 'url', None),
    'author': get_default(parser, 'package','author', None),
    'author_email': get_default(parser, 'package','author_email', None)
}

if parser.has_section('console_scripts'):
    scripts = [
        '{0} = {1}'.format(opt,  parser.get('console_scripts', opt))
        for opt in parser.options('console_scripts')
    ]
    setup_args['entry_points'] = {'console_scripts' : scripts}

if src_dir:
    setup_args['packages'] = setuptools.find_packages(src_dir, exclude=excl_dirs)
    setup_args['provides'] = setuptools.find_packages(src_dir)
    setup_args['package_dir'] = {'':src_dir}
setuptools.setup(**setup_args)




import sys
import json

if sys.version_info > (3, 0):
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError

    def urllib_request(method, url, headers, data):

        if isinstance(data, str):
            data = data.encode("utf-8")
        resp = urlopen(Request(method=method, url=url, headers=headers, data=data))
        return json.loads(resp.read().decode())

else:
    from urllib2 import urlopen, Request, HTTPError

    def urllib_request(method, url, headers, data):
        if isinstance(data, str):
            data = data.encode("utf-8")
        resp = urlopen(Request(url=url, headers=headers, data=data))
        return json.loads(resp.read().decode())
